#include "TraverseParallel.h"
#include "OpenMP.h"
#include "Mesh.h"

#ifdef _OPENMP

namespace AMDiS {

  TraverseParallelStack::TraverseParallelStack(int n, int mode)
    : nThreads(0),
      parallelMode(mode)
  {
    if (n == 0)
      nThreads = omp_get_overall_max_threads();
    else
      nThreads = n;
    
    stacks.resize(nThreads);

    if (parallelMode == 0) {
      int i = 0;
      for (std::vector<TraverseStack*>::iterator it = stacks.begin();
	   it != stacks.end(); ++it) {
	
	(*it) = new TraverseStack();
	(*it)->setMyThreadId(i++);
	(*it)->setMaxThreads(nThreads);
      }
    } else {
      for (std::vector<TraverseStack*>::iterator it = stacks.begin();
	   it != stacks.end(); ++it)	
	(*it) = new TraverseStack();      
    }
  }

  TraverseParallelStack::~TraverseParallelStack()
  {
    for (std::vector<TraverseStack*>::iterator it = stacks.begin();
	 it != stacks.end(); ++it)
      delete (*it);
  }

  ElInfo* TraverseParallelStack::traverseFirst(Mesh *mesh, int level, Flag fill_flag)
  {
    TEST_EXIT(fill_flag.isSet(Mesh::CALL_LEAF_EL))("not yet implemented");
    TEST_EXIT(stacks[omp_get_thread_num()])
      ("something wrong with parallel stack traverse");

    return stacks[omp_get_thread_num()]->traverseFirst(mesh, level, fill_flag);
  }
}

#endif // _OPENMP
